import sys
import os
import json
import shutil

import bupytest
import utoken

sys.path.insert(0, './')
from tea import tea


class TestTea(bupytest.UnitTest):
    def __init__(self):
        super().__init__()

        self.tea = tea.Tea()
        
        if os.path.isdir('tests/tea-test'):
            shutil.rmtree('tests/tea-test')

        if os.path.isdir('.tea'):
            shutil.rmtree('.tea')

        os.mkdir('tests/tea-test')

        with open('tests/tea-test/01.txt', 'w') as writer:
            writer.write('Hello world!')

        with open('tests/tea-test/02.txt', 'w') as writer:
            writer.write('Good morning.\n\nHow are you?')

    def test_create_repository(self):
        author = 'Lizard'
        author_email = 'someone@example.com'

        self.tea.create_repository(author, author_email)

        self.assert_true(os.path.isdir('.tea'))
        self.assert_true(os.path.isfile('.tea/.info'))

        repository_info = self.tea._get_repo_info()
        author_info = self.tea.get_author_info()

        self.assert_expected(author_info['author'], author)
        self.assert_expected(author_info['author_email'], author_email)
        self.assert_expected(repository_info['remote'], None)
        self.assert_expected(len(repository_info['key']), 64)

    def test_track_file(self):
        self.tea.track_file('tests/tea-test/01.txt')
        self.tea.track_file('tests/tea-test/02.txt')

        tracked_files = self.tea.get_tracked_files()

        self.assert_expected(
            value=tracked_files['./tests/tea-test/01.txt'],
            expected={
                'filename': './tests/tea-test/01.txt',
                'commits': []
            }
        )

        self.assert_expected(
            value=tracked_files['./tests/tea-test/02.txt'],
            expected={
                'filename': './tests/tea-test/02.txt',
                'commits': []
            }
        )

    def test_index_file_lines(self):
        file_lines_1 = self.tea.index_file_lines('tests/tea-test/01.txt')
        file_lines_2 = self.tea.index_file_lines('tests/tea-test/02.txt')

        self.assert_expected(
            value=file_lines_1,
            expected={0: 'Hello world!'}
        )

        self.assert_expected(
            value=file_lines_2,
            expected={
                0: 'Good morning.',
                1: '',
                2: 'How are you?'
            }
        )

    def test_get_lines_difference(self):
        file_lines_1 = self.tea.index_file_lines('tests/tea-test/01.txt')
        file_lines_2 = self.tea.index_file_lines('tests/tea-test/02.txt')

        file_lines_01 = {
            0: 'Hello world!',
            1: 'You are a programmer?'
        }

        file_lines_02 = {
            0: 'Good morning.',
            1: 'How are you?'
        }

        lines_01_diff = self.tea.get_lines_difference(file_lines_1, file_lines_01)
        lines_02_diff = self.tea.get_lines_difference(file_lines_2, file_lines_02)

        self.assert_expected(
            value=lines_01_diff,
            expected={1: 'You are a programmer?'}
        )

        self.assert_expected(
            value=lines_02_diff,
            expected={
                1: 'How are you?',
                2: None
            }
        )

    def test_commit(self):
        self.tea.commit(['tests/tea-test/01.txt', 'tests/tea-test/02.txt'], 'first commit')

        pieces_refs = self.tea._get_pieces_refs()
        repo_info = self.tea._get_repo_info()
        tracked_files = pieces_refs['tracked']
        commits = pieces_refs['commits']

        file_01_commits = tracked_files['./tests/tea-test/01.txt']['commits']
        file_02_commits = tracked_files['./tests/tea-test/02.txt']['commits']
        
        self.assert_expected(len(file_01_commits), 1)
        self.assert_expected(len(file_01_commits), 1)
        self.assert_expected(file_01_commits[0], file_02_commits[0])

        commit_info = commits[file_01_commits[0]]

        self.assert_expected(commit_info['author'], 'Lizard')
        self.assert_expected(commit_info['author_email'], 'someone@example.com')
        self.assert_expected(commit_info['message'], 'first commit')

        piece_id_01 = commit_info['files']['./tests/tea-test/01.txt']
        piece_id_02 = commit_info['files']['./tests/tea-test/02.txt']

        with open(f'.tea/pieces/{piece_id_01}', 'r') as reader:
            piece_token_01 = reader.read()

        with open(f'.tea/pieces/{piece_id_02}', 'r') as reader:
            piece_token_02 = reader.read()

        piece_info_01 = utoken.decode(piece_token_01, repo_info['key'])
        piece_info_02 = utoken.decode(piece_token_02, repo_info['key'])

        piece_lines_01 = piece_info_01['lines']
        piece_lines_02 = piece_info_02['lines']

        self.assert_expected(
            value=piece_lines_01,
            expected={'0': 'Hello world!'}
        )

        self.assert_expected(
            value=piece_lines_02,
            expected={
                '0': 'Good morning.',
                '1': '',
                '2': 'How are you?'
            }
        )

    def test_commit_2(self):
        with open('tests/tea-test/01.txt', 'w') as writer:
            writer.write('Hello world!\nHow are you?')

        self.tea.commit(['tests/tea-test/01.txt'], 'adding new line')

        pieces_refs = self.tea._get_pieces_refs()
        repo_info = self.tea._get_repo_info()
        tracked_files = pieces_refs['tracked']
        commits = pieces_refs['commits']

        file_01_commits = tracked_files['./tests/tea-test/01.txt']['commits']
        
        self.assert_expected(len(file_01_commits), 2)
        commit_info = commits[file_01_commits[1]]

        self.assert_expected(commit_info['author'], 'Lizard')
        self.assert_expected(commit_info['author_email'], 'someone@example.com')
        self.assert_expected(commit_info['message'], 'adding new line')

        piece_id_01 = commit_info['files']['./tests/tea-test/01.txt']

        with open(f'.tea/pieces/{piece_id_01}', 'r') as reader:
            piece_token_01 = reader.read()

        piece_info_01 = utoken.decode(piece_token_01, repo_info['key'])
        piece_lines_01 = piece_info_01['lines']

        # right below it is checked what has
        # been changed from the file.
        self.assert_expected(
            value=piece_lines_01,
            expected={
                '1': 'How are you?'
            }
        )

    def test_commit_3(self):
        with open('tests/tea-test/02.txt', 'w') as writer:
            writer.write('Good morning.\nHow are you?')

        self.tea.commit(['tests/tea-test/02.txt'], 'adding new line')

        pieces_refs = self.tea._get_pieces_refs()
        repo_info = self.tea._get_repo_info()
        tracked_files = pieces_refs['tracked']
        commits = pieces_refs['commits']

        file_02_commits = tracked_files['./tests/tea-test/02.txt']['commits']
        
        self.assert_expected(len(file_02_commits), 2)
        commit_info = commits[file_02_commits[1]]

        self.assert_expected(commit_info['author'], 'Lizard')
        self.assert_expected(commit_info['author_email'], 'someone@example.com')
        self.assert_expected(commit_info['message'], 'adding new line')

        piece_id_02 = commit_info['files']['./tests/tea-test/02.txt']

        with open(f'.tea/pieces/{piece_id_02}', 'r') as reader:
            piece_token_02 = reader.read()

        piece_info_02 = utoken.decode(piece_token_02, repo_info['key'])
        piece_lines_02 = piece_info_02['lines']

        # right below it is checked what has
        # been changed from the file.
        self.assert_expected(
            value=piece_lines_02,
            expected={
                '1': 'How are you?',
                '2': None
            }
        )

    def test_get_commits(self):
        commits = self.tea.get_commits()        
        self.assert_expected(len(commits), 3)

    def test_get_files_status(self):
        status = self.tea.get_files_status()
        self.assert_expected(
            value=status,
            expected=[
                {
                    'filepath': './LICENSE',
                    'status': 'untracked'
                },
                {
                    'filepath': './.ignore',
                    'status': 'untracked'
                }
            ]
        )

        self.tea.track_file('.ignore')
        self.tea.track_file('LICENSE')
        
        status = self.tea.get_files_status()
        self.assert_expected(
            value=status,
            expected=[
                {
                    'filepath': './.ignore',
                    'status': 'new'
                },
                {
                    'filepath': './LICENSE',
                    'status': 'new'
                }
            ]
        )

        self.tea.commit(['.ignore', 'LICENSE'], 'adding LICENSE')

        with open('tests/tea-test/01.txt', 'w') as writer:
            writer.write('Hello world!')

        with open('tests/tea-test/03.txt', 'w') as writer:
            writer.write('This is Tea')

        self.tea.track_file('tests/tea-test/03.txt')
        status = self.tea.get_files_status()

        self.assert_expected(
            value=status,
            expected=[
                {
                    'filepath': './tests/tea-test/01.txt',
                    'status': 'uncommitted'
                },
                {
                    'filepath': './tests/tea-test/03.txt',
                    'status': 'new'
                }
            ]
        )

        self.tea.commit(['tests/tea-test/01.txt'], 'removing lines')

        status = self.tea.get_files_status()
        self.assert_expected(
            value=status,
            expected=[
                {
                    'filepath': './tests/tea-test/03.txt',
                    'status': 'new'
                }
            ]
        )

        self.tea.commit(['tests/tea-test/03.txt'], 'adding new file')

        status = self.tea.get_files_status()
        self.assert_expected(status, [])

    def test_merge_files(self):
        with open('tests/tea-test/01.txt', 'w') as writer:
            writer.write('This is 01.txt file')

        with open('tests/tea-test/02.txt', 'w') as writer:
            writer.write('This is 02.txt file')

        with open('tests/tea-test/03.txt', 'w') as writer:
            writer.write('This is 03.txt file')

        merged = self.tea.merge()

        self.assert_true('./tests/tea-test/03.txt' in merged)
        self.assert_true('./tests/tea-test/02.txt' in merged)
        self.assert_true('./tests/tea-test/01.txt' in merged)
        self.assert_expected(len(merged), 3)

        status = self.tea.get_files_status()
        self.assert_expected(status, [])

if __name__ == '__main__':
    bupytest.this()
